/*
  An object of class HMCOutput implements the following public methods:

  Initialization:

	  HMCOutput(PrintStream);	Builds HMCOutput object from given stream.
	  HMCOutput();		Builds HMCOutput object from System.in

  HMCSupport.out comes pre-connected to System.out.


  Usage:

    print(x);	Prints the value of x on the stream. The exact 
			    output will depend on the values of various
			    switches described below. Takes values of any
			    type / class.

	println(x);	As above, but followed by a line break.

	println();	Send a line break to the stream.

	flush();	Force the stream to flush output buffers.


  Switches: All of these switches keep their values until explicitly changed.
	         This differs from some of the corresponding stream functions in C++
	         

	setAlwaysFlush(boolean); If set to true, all calls to print
				and println will be immediately followed
				by a flushing of the output buffer. 
				Default is true.	

	setWidth(int); 		Output of subsequent calls to print methods
				will be formatted in a field that many
				characters wide, if they are not already 
				wider than that. Default value is 0.
	
	setPadChar(char); 	Sets the character used to pad out fields.
				Default value is ' '.

	setJustification(int);  Sets the justification within a field.
				Default value is RIGHT. Other excepted values
				are LEFT and CENTER. Any other value
				behaves like RIGHT.

	setLeftJustify()	Sets left justification.
	setRightJustify()	Sets right justification.
	setCenterJustify()	Sets center justification.

	setPrecision(int);	Sets the number of digits to the right of
				the decimal point to which floating point
				values should be truncated. If set to 0,
				truncation does not occur. Default is 0.

	setFixed(boolean);	If set to true, then floating point
				values will be zero-extended on the right
				to yield the current precision setting.
				Default is false.

	     Note that the last two switches, which control formatting 
	     of floating point values, only apply if the entire argument
	     to print (or println) is such a value, since in a call like:

			println("x = " + x);

	     a string representation of x is generated by toString()
	     and println recieves a String, not a number. 
	     
  By Z. Dodds, based on code by J Hodas...
  
  It may seem that all of the different versions of print and println
  aren't needed, but they are -- otherwise the individual types are not handled
  correctly, e.g., setPrecision and setWidth information is lost...
*/	

package H;

import H.*;
import java.io.*;
import java.lang.*;

public class HMCOutput 
{
  static final int RIGHT = 1;
  static final int LEFT = 2;
  static final int CENTER = 3;

  private PrintStream theStream;
  private int fieldWidth;
  private int precision;
  private boolean fixed;
  private boolean alwaysFlush;
  private char padChar;
  private int justification;

  public HMCOutput(PrintStream O)
    {
      theStream = O;
      fieldWidth = 0;
      precision = 0;
      fixed = false;
      alwaysFlush = true;
      padChar = ' ';
      justification = RIGHT;
    }

  public HMCOutput()
    {
      this(System.out);
    }


  public void setWidth(int w)
    {
      fieldWidth = w;
    }

  public void setPadChar(char c)
    {
      padChar = c;
    }

  public void setJustification(int j)
    {
      justification = j;
    }

  public void setLeftJustify()
    {
      setJustification(LEFT);
    }

  public void setRightJustify()
    {
      setJustification(RIGHT);
    }

  public void setCenterJustify()
    {
      setJustification(CENTER);
    }

  public void setPrecision(int p)
    {
      precision = p;
    }

  public void setFixed(boolean f)
    {
      fixed = f;
    }

  public void setAlwaysFlush(boolean f)
    {
      alwaysFlush = f;
    }


  private void printString(String S, boolean newLine)
    {
      while (S.length() < fieldWidth)
	{
	  switch (justification) {
	  case RIGHT:
	    S = new String(padChar + S);
	    break;
	  case LEFT:
	    S = new String(S + padChar);
	    break;
	  case CENTER:
	    S = new String(S + padChar);
	    if (S.length() < fieldWidth)
	      S = new String(padChar + S);
	    break;
	  default:		// default behavior is right justification
	    S = new String(padChar + S);
	    break;
	  }
	}

      if (newLine)
	theStream.println(S);
      else 
	theStream.print(S);

      if (alwaysFlush)
	theStream.flush();
    }

  private void printLong(long l, boolean newLine)
    {
      printString(String.valueOf(l), newLine);
    }

  private void printDouble(double d, boolean newLine)
  {

    String dString = Double.toString(d);

    int eLocation = dString.lastIndexOf('e');

    if (eLocation == -1) 
      eLocation = dString.lastIndexOf('E');

    if (eLocation == -1) 
      eLocation = dString.length();

    String exponent = dString.substring(eLocation);
    String core = dString.substring(0,eLocation);

    int dotLocation = core.lastIndexOf('.');

    String intPart;
    String decPart;
    if (dotLocation == -1) {
      intPart = core;
      decPart = "";
    }
    else {
      intPart = core.substring(0,dotLocation);
      decPart = core.substring(dotLocation+1);
    }

    if (precision != 0 && decPart.length() > precision)
      decPart = decPart.substring(0,precision);

    if (exponent.length() > 0 && decPart.length() == 0)
      decPart = "0";

    if (fixed)
      while (decPart.length() < precision)
        decPart = decPart + "0";

    String theNumber;
    if (decPart.length() > 0)
      theNumber = intPart + "." + decPart + exponent;
    else
      theNumber = intPart;

    printString(theNumber, newLine);
  }
  
  public void print(boolean b)
  {
    printString(b ? "true" : "false",false);
  }  
  
  public void print(byte b)
  {
    printLong((long)b,false);
  } 
  
  public void print(Byte b)
  {
    printLong(b.longValue(),false);
  }   
  
  public void print(short s)
  {
    printLong((long)s,false);
  } 
  
  public void print(Short s)
  {
    printLong(s.longValue(),false);
  }   

  public void print(long l)
  {
    printLong(l,false);
  }

  public void print(Long L)
  {
    printLong(L.longValue(),false);
  }

  public void print(int i)
  {
    printLong((long) i,false);
  }

  public void print(Integer I)
  {
    printLong(I.longValue(),false);
  }

  public void print(double d)
  {
    printDouble(d,false);
  }

  public void print(Double D)
  {
    printDouble(D.doubleValue(),false);
  }

  public void print(float f)
  {
    printDouble((double) f,false);
  }

  public void print(Float F)
  {
    printDouble(F.doubleValue(),false);
  }

  public void print(String S)
  {
    printString(S,false);
  }

  public void print(char c)
  {
    printString(String.valueOf(c),false);
  }

  public void print(Object O)
  {
    printString(O.toString(),false);
  }
  
  /*
   * println functions
   */
  
  public void println(boolean b)
  {
    printString(b ? "true" : "false",true);
  }  
  
  public void println(byte b)
  {
    printLong((long)b,true);
  } 
  
  public void println(Byte b)
  {
    printLong(b.longValue(),true);
  }   
  
  public void println(short s)
  {
    printLong((long)s,true);
  } 
  
  public void println(Short s)
  {
    printLong(s.longValue(),true);
  }  

  public void println(long l)
  {
    printLong(l,true);
  }

  public void println(Long L)
  {
    printLong(L.longValue(),true);
  }

  public void println(int i)
  {
    printLong((long) i,true);
  }

  public void println(Integer I)
  {
    printLong(I.longValue(),true);
  }

  public void println(double d)
  {
    printDouble(d,true);
  }

  public void println(Double D)
  {
    printDouble(D.doubleValue(),true);
  }

  public void println(float f)
  {
    printDouble((double) f,true);
  }

  public void println(Float F)
  {
    printDouble(F.doubleValue(),true);
  }

  public void println(String S)
  {
    printString(S,true);
  }

  public void println(char c)
  {
    printString(String.valueOf(c),true);
  }

  public void println(Object O)
  {
    printString(O.toString(),true);
  }

  public void println() 
  {
    theStream.println();

    if (alwaysFlush)
      theStream.flush();
  }

  public void flush()
  {
    theStream.flush();
  }


  static public void main(String[] args)
  {
    ;
  }

}

